"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.injectFunctionNames = exports.FunctionNames = void 0;

var _serverlessPluginCustomRoles = _interopRequireDefault(require("serverless-plugin-custom-roles"));

var _iam = require("./iam");

var _layers = require("./layers");

var _tables = require("./tables");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const FunctionNames = {
  HTTP: 'graphQlHttp',
  WEBSOCKET: 'graphQlWebsocket',
  EVENT: 'graphQlEventProcessor'
};
exports.FunctionNames = FunctionNames;

const getHandlerPath = handlerName => `node_modules/graphql-lambda-ws/lib/index.${handlerName}`;

const injectFunctionNames = plugin => {
  if (!plugin.service.functions) {
    // eslint-disable-next-line no-param-reassign
    plugin.service.functions = {};
  }

  plugin.verbose('Injecting function names');
  Object.assign(plugin.service.functions, {
    [FunctionNames.HTTP]: {
      handler: getHandlerPath('httpHandler')
    },
    [FunctionNames.WEBSOCKET]: {
      handler: getHandlerPath('websocketHandler')
    },
    [FunctionNames.EVENT]: {
      handler: getHandlerPath('eventHandler')
    }
  });
};

exports.injectFunctionNames = injectFunctionNames;

const processFunctions = async plugin => {
  plugin.verbose('Processing functions');

  if (!plugin.service.functions) {
    // eslint-disable-next-line no-param-reassign
    plugin.service.functions = {};
  }

  const {
    iamRoleStatements = [],
    httpFunctionOptions = {},
    httpOptions = {},
    wsFunctionOptions = {},
    authorizer
  } = plugin.getOptions();
  const layers = [(0, _layers.getLayerName)(plugin)];
  const sharedGraphQlIam = [...iamRoleStatements, (0, _iam.createPolicy)(['dynamodb:PutItem'], {
    'Fn::GetAtt': [_tables.ResourceNames.EVENTS, 'Arn']
  })];
  const stage = plugin.getStage();
  Object.assign(plugin.service.functions, {
    [FunctionNames.HTTP]: _objectSpread({
      handler: getHandlerPath('httpHandler'),
      description: 'Lambda that handles graphQL requests over HTTP',
      layers
    }, httpFunctionOptions, {
      events: [{
        http: _objectSpread({
          path: '/graphql',
          method: 'post'
        }, httpOptions, {
          authorizer: authorizer || httpOptions.authorizer
        })
      }],
      iamRoleStatements: [...sharedGraphQlIam]
    }),
    [FunctionNames.WEBSOCKET]: _objectSpread({
      handler: getHandlerPath('websocketHandler'),
      description: 'Lambda that handles graphQL requests over web sockets',
      layers
    }, wsFunctionOptions, {
      events: [{
        websocket: {
          route: '$connect',
          authorizer
        }
      }, {
        websocket: '$disconnect'
      }, {
        websocket: '$default'
      }],
      iamRoleStatements: [...sharedGraphQlIam, (0, _iam.createWebSocketPolicy)(stage), (0, _iam.createPolicy)(['dynamodb:DeleteItem', 'dynamodb:GetItem', 'dynamodb:PutItem'], {
        'Fn::GetAtt': [_tables.ResourceNames.CONNECTIONS, 'Arn']
      }), (0, _iam.createPolicy)(['dynamodb:BatchWriteItem', 'dynamodb:DeleteItem', 'dynamodb:GetItem', 'dynamodb:PutItem'], {
        'Fn::GetAtt': [_tables.ResourceNames.SUBSCRIPTIONS, 'Arn']
      }), (0, _iam.createPolicy)(['dynamodb:BatchWriteItem', 'dynamodb:DeleteItem', 'dynamodb:GetItem', 'dynamodb:PutItem'], {
        'Fn::GetAtt': [_tables.ResourceNames.SUBSCRIPTION_OPS, 'Arn']
      })]
    }),
    [FunctionNames.EVENT]: {
      handler: getHandlerPath('eventHandler'),
      description: 'Lambda that notifies subscribers when events are published',
      events: [{
        stream: {
          type: 'dynamodb',
          arn: {
            'Fn::GetAtt': [_tables.ResourceNames.EVENTS, 'StreamArn']
          },
          batchSize: 100,
          startingPosition: 'LATEST'
        }
      }],
      timeout: 300,
      memorySize: 1024,
      layers,
      iamRoleStatements: [(0, _iam.createWebSocketPolicy)(stage), (0, _iam.createPolicy)(['dynamodb:DescribeStream', 'dynamodb:GetRecords', 'dynamodb:GetShardIterator', 'dynamodb:ListStreams'], {
        'Fn::GetAtt': [_tables.ResourceNames.EVENTS, 'StreamArn']
      }), (0, _iam.createPolicy)(['dynamodb:Query'], {
        'Fn::GetAtt': [_tables.ResourceNames.SUBSCRIPTIONS, 'Arn']
      }), (0, _iam.createPolicy)(['dynamodb:DeleteItem'], {
        'Fn::GetAtt': [_tables.ResourceNames.CONNECTIONS, 'Arn']
      }), (0, _iam.createPolicy)(['dynamodb:BatchWriteItem', 'dynamodb:DeleteItem', 'dynamodb:GetItem', 'dynamodb:PutItem'], {
        'Fn::GetAtt': [_tables.ResourceNames.SUBSCRIPTION_OPS, 'Arn']
      })]
    }
  });
  plugin.verbose('Initializing functions');
  plugin.service.setFunctionNames(plugin.options);
  plugin.verbose('Processing function roles');
  const customRolesPlugin = new _serverlessPluginCustomRoles.default(plugin.serverless, plugin.options);
  await customRolesPlugin.createRoles();
};

var _default = processFunctions;
exports.default = _default;