"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createWebSocketPolicy = exports.createPolicy = void 0;

const createPolicy = (actions, ...resources) => {
  const arrayActions = Array.isArray(actions) ? actions : [actions];
  return {
    Effect: 'Allow',
    Action: arrayActions,
    Resource: resources
  };
};

exports.createPolicy = createPolicy;

const createWebSocketPolicy = stage => {
  // https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-control-access-iam.html
  const actions = 'execute-api:ManageConnections';
  const resources = {
    'Fn::Join': [':', ['arn:aws:execute-api', {
      Ref: 'AWS::Region'
    }, {
      Ref: 'AWS::AccountId'
    }, `*/${stage}/*/@connections/*`]]
  };
  return createPolicy(actions, resources);
};

exports.createWebSocketPolicy = createWebSocketPolicy;