"use strict";

var _tables = _interopRequireDefault(require("./tables"));

var _layers = _interopRequireDefault(require("./layers"));

var _functions = _interopRequireWildcard(require("./functions"));

var _provider = _interopRequireDefault(require("./provider"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const PLUGIN_NAME = 'graphql-lambda-ws';
const PACKAGE_NAME = `serverless-plugin-${PLUGIN_NAME}`;

class GraphQlLambdaWsPlugin {
  /* eslint-disable flowtype/no-flow-fix-me-comments, no-undef */
  // eslint-disable-line flowtype/no-weak-types
  // eslint-disable-line flowtype/no-weak-types

  /* eslint-enable flowtype/no-flow-fix-me-comments */
  // eslint-disable-next-line flowtype/no-flow-fix-me-comments
  constructor(serverless, options) {
    _defineProperty(this, "serverless", void 0);

    _defineProperty(this, "options", void 0);

    _defineProperty(this, "provider", void 0);

    _defineProperty(this, "service", void 0);

    _defineProperty(this, "isVerbose", void 0);

    _defineProperty(this, "isDebug", void 0);

    _defineProperty(this, "hooks", void 0);

    _defineProperty(this, "commands", void 0);

    this.serverless = serverless;
    this.options = options;
    this.provider = this.serverless.getProvider('aws');
    this.service = serverless.service;
    this.isVerbose = options.verbose || options.v;
    this.isDebug = options.debug || options.d;
    this.commands = {
      [PLUGIN_NAME]: {
        usage: 'Add graphQL subscription support via API Gateway web sockets',
        lifecycleEvents: [PLUGIN_NAME],
        options: {
          verbose: {
            usage: 'Verbose output',
            shortcut: 'v'
          },
          debug: {
            usage: 'Debug the plugin',
            shortcut: 'd'
          }
        },
        commands: {
          process: {
            usage: 'Process the plugin',
            lifecycleEvents: ['process']
          }
        }
      }
    };
    this.hooks = {
      [`${PLUGIN_NAME}:process:process`]: () => this.process(),
      'before:package:setupProviderConfiguration': () => this.spawn('process')
    }; // Need to inject function names early to support function invokes

    (0, _functions.injectFunctionNames)(this);
  }

  log(...args) {
    const message = args.join(' ');
    this.serverless.cli.consoleLog(`${PACKAGE_NAME}: ${message}`);
  }

  verbose(...args) {
    if (!this.isVerbose && !this.isDebug) {
      return;
    }

    this.log(...args);
  }

  debug(...args) {
    if (!this.isDebug) {
      return;
    }

    this.log(...args);
  }

  spawn(command) {
    return this.serverless.pluginManager.spawn(`${PLUGIN_NAME}:${command}`);
  }

  getOptions() {
    return this.serverless.service.custom[PLUGIN_NAME];
  }

  getStage() {
    return this.options.stage || this.service.provider.stage;
  }

  async process() {
    try {
      const processes = [_provider.default, _layers.default, _functions.default, _tables.default];
      processes.reduce(async (acc, p) => {
        // eslint-disable-next-line no-param-reassign
        acc = await p(this, acc);
        return acc;
      }, {});
    } catch (err) {
      if (this.isDebug) {
        // Use console to log out stack trace
        console.error('ERROR', err); // eslint-disable-line no-console
      }

      throw err;
    }
  }

}

module.exports = GraphQlLambdaWsPlugin; // $FlowIgnoreLine

Object.assign(module.exports, {
  PLUGIN_NAME,
  PACKAGE_NAME
});