"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.getLayerName = void 0;

var _pascalCase = _interopRequireDefault(require("pascal-case"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const AWS_LATEST_LAYER_NAME = 'awsLatest';

const getLayerName = plugin => {
  const {
    awsLayer
  } = plugin.getOptions();

  if (awsLayer) {
    // Return supplied layer data
    return awsLayer;
  }

  return {
    Ref: `${(0, _pascalCase.default)(AWS_LATEST_LAYER_NAME)}LambdaLayer`
  };
};

exports.getLayerName = getLayerName;

const processLayers = async plugin => {
  plugin.verbose('Processing layers');
  const {
    awsLayer
  } = plugin.getOptions();

  if (awsLayer) {
    plugin.verbose('awsLayer supplied', awsLayer);
    return;
  }

  if (!plugin.service.layers) {
    // eslint-disable-next-line no-param-reassign
    plugin.service.layers = {};
  }

  const stage = plugin.getStage();
  Object.assign(plugin.service.layers, {
    [AWS_LATEST_LAYER_NAME]: {
      name: `${stage}-aws-latest`,
      description: 'Latest aws sdk for node',
      path: 'node_modules/serverless-plugin-graphql-lambda-ws/layers/aws-latest',
      compatibleRuntimes: ['nodejs6.10', 'nodejs8.10']
    }
  });
};

var _default = processLayers;
exports.default = _default;