"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.createPluginTableNames = exports.ResourceNames = void 0;

var _serverlessPluginTables = _interopRequireDefault(require("serverless-plugin-tables"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const createTableName = (plugin, name) => {
  const serviceName = plugin.service.getServiceName();
  const stage = plugin.options.stage || plugin.service.provider.stage;
  return `${serviceName}-${stage}-${name}`;
};

const ResourceNames = {
  SUBSCRIPTIONS: 'GraphQlSubscriptionsDynamoDBTable',
  CONNECTIONS: 'GraphQlConnectionsDynamoDBTable',
  EVENTS: 'GraphQlEventsDynamoDBTable',
  SUBSCRIPTION_OPS: 'GraphQlSubscriptionOpsDynamoDBTable'
};
exports.ResourceNames = ResourceNames;
const TableNames = {
  SUBSCRIPTIONS: 'GraphQlSubscriptions',
  CONNECTIONS: 'GraphQlConnections',
  EVENTS: 'GraphQlEvents',
  SUBSCRIPTION_OPS: 'GraphQlSubscriptionOps'
};

const createPluginTableNames = plugin => {
  const {
    tableNames = {}
  } = plugin.getOptions();
  return {
    SUBSCRIPTIONS: tableNames.subscriptions || createTableName(plugin, TableNames.SUBSCRIPTIONS),
    CONNECTIONS: tableNames.connections || createTableName(plugin, TableNames.CONNECTIONS),
    EVENTS: tableNames.events || createTableName(plugin, TableNames.EVENTS),
    SUBSCRIPTION_OPS: tableNames.subscriptionOperations || createTableName(plugin, TableNames.SUBSCRIPTION_OPS)
  };
};

exports.createPluginTableNames = createPluginTableNames;

const processTables = async plugin => {
  plugin.verbose('Processing table resources');

  if (!plugin.service.resources) {
    // eslint-disable-next-line no-param-reassign
    plugin.service.resources = {};
  }

  const existingTables = plugin.service.resources.tables || {};
  const pluginTableNames = createPluginTableNames(plugin);
  Object.assign(plugin.service.resources, {
    tables: _objectSpread({}, existingTables, {
      GraphQlConnections: {
        name: pluginTableNames.CONNECTIONS,
        resourceName: ResourceNames.CONNECTIONS,
        partitionKey: 'id'
      },
      GraphQlSubscriptions: {
        name: pluginTableNames.SUBSCRIPTIONS,
        resourceName: ResourceNames.SUBSCRIPTIONS,
        partitionKey: 'event',
        sortKey: 'subscriptionId'
      },
      GraphQlEventsDynamoDBTable: {
        name: pluginTableNames.EVENTS,
        resourceName: ResourceNames.EVENTS,
        partitionKey: 'id',
        streamType: 'newItem'
      },
      GraphQlSubscriptionsOpDynamoDBTable: {
        name: pluginTableNames.SUBSCRIPTION_OPS,
        resourceName: ResourceNames.EVENTS,
        partitionKey: 'subscriptionId'
      }
    })
  });
  const tablesPlugin = new _serverlessPluginTables.default(plugin.serverless, plugin.options);
  await tablesPlugin.process();
};

var _default = processTables;
exports.default = _default;